import { ChainId, ChainName, Network } from "../../";
import { BigNumber, ethers } from "ethers";
import { RelayerPayloadId, DeliveryInstruction, VaaKey } from "../structs";
import { DeliveryTargetInfo } from "./helpers";
export declare type InfoRequestParams = {
    environment?: Network;
    sourceChainProvider?: ethers.providers.Provider;
    targetChainProviders?: Map<ChainName, ethers.providers.Provider>;
    targetChainBlockRanges?: Map<ChainName, [
        ethers.providers.BlockTag,
        ethers.providers.BlockTag
    ]>;
    wormholeRelayerWhMessageIndex?: number;
    wormholeRelayerAddresses?: Map<ChainName, string>;
};
export declare type DeliveryInfo = {
    type: RelayerPayloadId.Delivery;
    sourceChain: ChainName;
    sourceTransactionHash: string;
    sourceDeliverySequenceNumber: number;
    deliveryInstruction: DeliveryInstruction;
    targetChainStatus: {
        chain: ChainName;
        events: DeliveryTargetInfo[];
    };
};
export declare function printWormholeRelayerInfo(info: DeliveryInfo): void;
export declare function stringifyWormholeRelayerInfo(info: DeliveryInfo): string;
export declare type SendOptionalParams = {
    environment?: Network;
    receiverValue?: ethers.BigNumberish;
    paymentForExtraReceiverValue?: ethers.BigNumberish;
    additionalVaas?: [
        {
            chainId?: ChainId;
            emitterAddress: string;
            sequenceNumber: ethers.BigNumberish;
        }
    ];
    deliveryProviderAddress?: string;
    consistencyLevel?: ethers.BigNumberish;
    refundChainId?: ChainId;
    refundAddress?: string;
    relayParameters?: ethers.BytesLike;
};
export declare function sendToEvm(signer: ethers.Signer, sourceChain: ChainName, targetChain: ChainName, targetAddress: string, payload: ethers.BytesLike, gasLimit: BigNumber | number, overrides?: ethers.PayableOverrides, sendOptionalParams?: SendOptionalParams): Promise<ethers.providers.TransactionResponse>;
export declare type GetPriceOptParams = {
    environment?: Network;
    receiverValue?: ethers.BigNumberish;
    deliveryProviderAddress?: string;
    sourceChainProvider?: ethers.providers.Provider;
};
export declare function getPriceAndRefundInfo(sourceChain: ChainName, targetChain: ChainName, gasAmount: ethers.BigNumberish, optionalParams?: GetPriceOptParams): Promise<[ethers.BigNumber, ethers.BigNumber]>;
export declare function getPrice(sourceChain: ChainName, targetChain: ChainName, gasAmount: ethers.BigNumberish, optionalParams?: GetPriceOptParams): Promise<ethers.BigNumber>;
export declare function getWormholeRelayerInfo(sourceChain: ChainName, sourceTransaction: string, infoRequest?: InfoRequestParams): Promise<DeliveryInfo>;
export declare function resendRaw(signer: ethers.Signer, sourceChain: ChainName, targetChain: ChainName, environment: Network, vaaKey: VaaKey, newGasLimit: BigNumber | number, newReceiverValue: BigNumber | number, deliveryProviderAddress: string, overrides?: ethers.PayableOverrides): Promise<ethers.ContractTransaction>;
export declare function resend(signer: ethers.Signer, sourceChain: ChainName, targetChain: ChainName, environment: Network, vaaKey: VaaKey, newGasLimit: BigNumber | number, newReceiverValue: BigNumber | number, deliveryProviderAddress: string, wormholeRPCs: string[], overrides: ethers.PayableOverrides, isNode?: boolean): Promise<ethers.ContractTransaction>;
export declare function getVAA(wormholeRPCs: string[], vaaKey: VaaKey, isNode?: boolean): Promise<Uint8Array>;
