import type { BaseContract, BigNumber, BigNumberish, BytesLike, CallOverrides, ContractTransaction, Overrides, PopulatedTransaction, Signer, utils } from "ethers";
import type { FunctionFragment, Result } from "@ethersproject/abi";
import type { Listener, Provider } from "@ethersproject/providers";
import type { TypedEventFilter, TypedEvent, TypedListener, OnEvent, PromiseOrValue } from "../common";
export declare namespace Structs {
    type GuardianSetStruct = {
        keys: PromiseOrValue<string>[];
        expirationTime: PromiseOrValue<BigNumberish>;
    };
    type GuardianSetStructOutput = [string[], number] & {
        keys: string[];
        expirationTime: number;
    };
    type SignatureStruct = {
        r: PromiseOrValue<BytesLike>;
        s: PromiseOrValue<BytesLike>;
        v: PromiseOrValue<BigNumberish>;
        guardianIndex: PromiseOrValue<BigNumberish>;
    };
    type SignatureStructOutput = [string, string, number, number] & {
        r: string;
        s: string;
        v: number;
        guardianIndex: number;
    };
    type VMStruct = {
        version: PromiseOrValue<BigNumberish>;
        timestamp: PromiseOrValue<BigNumberish>;
        nonce: PromiseOrValue<BigNumberish>;
        emitterChainId: PromiseOrValue<BigNumberish>;
        emitterAddress: PromiseOrValue<BytesLike>;
        sequence: PromiseOrValue<BigNumberish>;
        consistencyLevel: PromiseOrValue<BigNumberish>;
        payload: PromiseOrValue<BytesLike>;
        guardianSetIndex: PromiseOrValue<BigNumberish>;
        signatures: Structs.SignatureStruct[];
        hash: PromiseOrValue<BytesLike>;
    };
    type VMStructOutput = [
        number,
        number,
        number,
        number,
        string,
        BigNumber,
        number,
        string,
        number,
        Structs.SignatureStructOutput[],
        string
    ] & {
        version: number;
        timestamp: number;
        nonce: number;
        emitterChainId: number;
        emitterAddress: string;
        sequence: BigNumber;
        consistencyLevel: number;
        payload: string;
        guardianSetIndex: number;
        signatures: Structs.SignatureStructOutput[];
        hash: string;
    };
}
export interface ExportedMessagesInterface extends utils.Interface {
    functions: {
        "chainId()": FunctionFragment;
        "evmChainId()": FunctionFragment;
        "getCurrentGuardianSetIndex()": FunctionFragment;
        "getGuardianSet(uint32)": FunctionFragment;
        "getGuardianSetExpiry()": FunctionFragment;
        "governanceActionIsConsumed(bytes32)": FunctionFragment;
        "governanceChainId()": FunctionFragment;
        "governanceContract()": FunctionFragment;
        "isFork()": FunctionFragment;
        "isInitialized(address)": FunctionFragment;
        "messageFee()": FunctionFragment;
        "nextSequence(address)": FunctionFragment;
        "parseAndVerifyVM(bytes)": FunctionFragment;
        "parseVM(bytes)": FunctionFragment;
        "quorum(uint256)": FunctionFragment;
        "storeGuardianSetPub((address[],uint32),uint32)": FunctionFragment;
        "verifySignatures(bytes32,(bytes32,bytes32,uint8,uint8)[],(address[],uint32))": FunctionFragment;
        "verifyVM((uint8,uint32,uint32,uint16,bytes32,uint64,uint8,bytes,uint32,(bytes32,bytes32,uint8,uint8)[],bytes32))": FunctionFragment;
    };
    getFunction(nameOrSignatureOrTopic: "chainId" | "evmChainId" | "getCurrentGuardianSetIndex" | "getGuardianSet" | "getGuardianSetExpiry" | "governanceActionIsConsumed" | "governanceChainId" | "governanceContract" | "isFork" | "isInitialized" | "messageFee" | "nextSequence" | "parseAndVerifyVM" | "parseVM" | "quorum" | "storeGuardianSetPub" | "verifySignatures" | "verifyVM"): FunctionFragment;
    encodeFunctionData(functionFragment: "chainId", values?: undefined): string;
    encodeFunctionData(functionFragment: "evmChainId", values?: undefined): string;
    encodeFunctionData(functionFragment: "getCurrentGuardianSetIndex", values?: undefined): string;
    encodeFunctionData(functionFragment: "getGuardianSet", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getGuardianSetExpiry", values?: undefined): string;
    encodeFunctionData(functionFragment: "governanceActionIsConsumed", values: [PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "governanceChainId", values?: undefined): string;
    encodeFunctionData(functionFragment: "governanceContract", values?: undefined): string;
    encodeFunctionData(functionFragment: "isFork", values?: undefined): string;
    encodeFunctionData(functionFragment: "isInitialized", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "messageFee", values?: undefined): string;
    encodeFunctionData(functionFragment: "nextSequence", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "parseAndVerifyVM", values: [PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "parseVM", values: [PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "quorum", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "storeGuardianSetPub", values: [Structs.GuardianSetStruct, PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "verifySignatures", values: [
        PromiseOrValue<BytesLike>,
        Structs.SignatureStruct[],
        Structs.GuardianSetStruct
    ]): string;
    encodeFunctionData(functionFragment: "verifyVM", values: [Structs.VMStruct]): string;
    decodeFunctionResult(functionFragment: "chainId", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "evmChainId", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getCurrentGuardianSetIndex", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getGuardianSet", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getGuardianSetExpiry", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "governanceActionIsConsumed", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "governanceChainId", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "governanceContract", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "isFork", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "isInitialized", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "messageFee", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "nextSequence", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "parseAndVerifyVM", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "parseVM", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "quorum", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "storeGuardianSetPub", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "verifySignatures", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "verifyVM", data: BytesLike): Result;
    events: {};
}
export interface ExportedMessages extends BaseContract {
    connect(signerOrProvider: Signer | Provider | string): this;
    attach(addressOrName: string): this;
    deployed(): Promise<this>;
    interface: ExportedMessagesInterface;
    queryFilter<TEvent extends TypedEvent>(event: TypedEventFilter<TEvent>, fromBlockOrBlockhash?: string | number | undefined, toBlock?: string | number | undefined): Promise<Array<TEvent>>;
    listeners<TEvent extends TypedEvent>(eventFilter?: TypedEventFilter<TEvent>): Array<TypedListener<TEvent>>;
    listeners(eventName?: string): Array<Listener>;
    removeAllListeners<TEvent extends TypedEvent>(eventFilter: TypedEventFilter<TEvent>): this;
    removeAllListeners(eventName?: string): this;
    off: OnEvent<this>;
    on: OnEvent<this>;
    once: OnEvent<this>;
    removeListener: OnEvent<this>;
    functions: {
        chainId(overrides?: CallOverrides): Promise<[number]>;
        evmChainId(overrides?: CallOverrides): Promise<[BigNumber]>;
        getCurrentGuardianSetIndex(overrides?: CallOverrides): Promise<[number]>;
        getGuardianSet(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[Structs.GuardianSetStructOutput]>;
        getGuardianSetExpiry(overrides?: CallOverrides): Promise<[number]>;
        governanceActionIsConsumed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[boolean]>;
        governanceChainId(overrides?: CallOverrides): Promise<[number]>;
        governanceContract(overrides?: CallOverrides): Promise<[string]>;
        isFork(overrides?: CallOverrides): Promise<[boolean]>;
        isInitialized(impl: PromiseOrValue<string>, overrides?: CallOverrides): Promise<[boolean]>;
        messageFee(overrides?: CallOverrides): Promise<[BigNumber]>;
        nextSequence(emitter: PromiseOrValue<string>, overrides?: CallOverrides): Promise<[BigNumber]>;
        parseAndVerifyVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[
            Structs.VMStructOutput,
            boolean,
            string
        ] & {
            vm: Structs.VMStructOutput;
            valid: boolean;
            reason: string;
        }>;
        parseVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[Structs.VMStructOutput] & {
            vm: Structs.VMStructOutput;
        }>;
        quorum(numGuardians: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[BigNumber] & {
            numSignaturesRequiredForQuorum: BigNumber;
        }>;
        storeGuardianSetPub(set: Structs.GuardianSetStruct, index: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        verifySignatures(hash: PromiseOrValue<BytesLike>, signatures: Structs.SignatureStruct[], guardianSet: Structs.GuardianSetStruct, overrides?: CallOverrides): Promise<[boolean, string] & {
            valid: boolean;
            reason: string;
        }>;
        verifyVM(vm: Structs.VMStruct, overrides?: CallOverrides): Promise<[boolean, string] & {
            valid: boolean;
            reason: string;
        }>;
    };
    chainId(overrides?: CallOverrides): Promise<number>;
    evmChainId(overrides?: CallOverrides): Promise<BigNumber>;
    getCurrentGuardianSetIndex(overrides?: CallOverrides): Promise<number>;
    getGuardianSet(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<Structs.GuardianSetStructOutput>;
    getGuardianSetExpiry(overrides?: CallOverrides): Promise<number>;
    governanceActionIsConsumed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
    governanceChainId(overrides?: CallOverrides): Promise<number>;
    governanceContract(overrides?: CallOverrides): Promise<string>;
    isFork(overrides?: CallOverrides): Promise<boolean>;
    isInitialized(impl: PromiseOrValue<string>, overrides?: CallOverrides): Promise<boolean>;
    messageFee(overrides?: CallOverrides): Promise<BigNumber>;
    nextSequence(emitter: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
    parseAndVerifyVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[
        Structs.VMStructOutput,
        boolean,
        string
    ] & {
        vm: Structs.VMStructOutput;
        valid: boolean;
        reason: string;
    }>;
    parseVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<Structs.VMStructOutput>;
    quorum(numGuardians: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
    storeGuardianSetPub(set: Structs.GuardianSetStruct, index: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    verifySignatures(hash: PromiseOrValue<BytesLike>, signatures: Structs.SignatureStruct[], guardianSet: Structs.GuardianSetStruct, overrides?: CallOverrides): Promise<[boolean, string] & {
        valid: boolean;
        reason: string;
    }>;
    verifyVM(vm: Structs.VMStruct, overrides?: CallOverrides): Promise<[boolean, string] & {
        valid: boolean;
        reason: string;
    }>;
    callStatic: {
        chainId(overrides?: CallOverrides): Promise<number>;
        evmChainId(overrides?: CallOverrides): Promise<BigNumber>;
        getCurrentGuardianSetIndex(overrides?: CallOverrides): Promise<number>;
        getGuardianSet(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<Structs.GuardianSetStructOutput>;
        getGuardianSetExpiry(overrides?: CallOverrides): Promise<number>;
        governanceActionIsConsumed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
        governanceChainId(overrides?: CallOverrides): Promise<number>;
        governanceContract(overrides?: CallOverrides): Promise<string>;
        isFork(overrides?: CallOverrides): Promise<boolean>;
        isInitialized(impl: PromiseOrValue<string>, overrides?: CallOverrides): Promise<boolean>;
        messageFee(overrides?: CallOverrides): Promise<BigNumber>;
        nextSequence(emitter: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        parseAndVerifyVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[
            Structs.VMStructOutput,
            boolean,
            string
        ] & {
            vm: Structs.VMStructOutput;
            valid: boolean;
            reason: string;
        }>;
        parseVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<Structs.VMStructOutput>;
        quorum(numGuardians: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        storeGuardianSetPub(set: Structs.GuardianSetStruct, index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        verifySignatures(hash: PromiseOrValue<BytesLike>, signatures: Structs.SignatureStruct[], guardianSet: Structs.GuardianSetStruct, overrides?: CallOverrides): Promise<[boolean, string] & {
            valid: boolean;
            reason: string;
        }>;
        verifyVM(vm: Structs.VMStruct, overrides?: CallOverrides): Promise<[boolean, string] & {
            valid: boolean;
            reason: string;
        }>;
    };
    filters: {};
    estimateGas: {
        chainId(overrides?: CallOverrides): Promise<BigNumber>;
        evmChainId(overrides?: CallOverrides): Promise<BigNumber>;
        getCurrentGuardianSetIndex(overrides?: CallOverrides): Promise<BigNumber>;
        getGuardianSet(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getGuardianSetExpiry(overrides?: CallOverrides): Promise<BigNumber>;
        governanceActionIsConsumed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        governanceChainId(overrides?: CallOverrides): Promise<BigNumber>;
        governanceContract(overrides?: CallOverrides): Promise<BigNumber>;
        isFork(overrides?: CallOverrides): Promise<BigNumber>;
        isInitialized(impl: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        messageFee(overrides?: CallOverrides): Promise<BigNumber>;
        nextSequence(emitter: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        parseAndVerifyVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        parseVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        quorum(numGuardians: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        storeGuardianSetPub(set: Structs.GuardianSetStruct, index: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        verifySignatures(hash: PromiseOrValue<BytesLike>, signatures: Structs.SignatureStruct[], guardianSet: Structs.GuardianSetStruct, overrides?: CallOverrides): Promise<BigNumber>;
        verifyVM(vm: Structs.VMStruct, overrides?: CallOverrides): Promise<BigNumber>;
    };
    populateTransaction: {
        chainId(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        evmChainId(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getCurrentGuardianSetIndex(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getGuardianSet(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getGuardianSetExpiry(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        governanceActionIsConsumed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        governanceChainId(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        governanceContract(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        isFork(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        isInitialized(impl: PromiseOrValue<string>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        messageFee(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        nextSequence(emitter: PromiseOrValue<string>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        parseAndVerifyVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        parseVM(encodedVM: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        quorum(numGuardians: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        storeGuardianSetPub(set: Structs.GuardianSetStruct, index: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        verifySignatures(hash: PromiseOrValue<BytesLike>, signatures: Structs.SignatureStruct[], guardianSet: Structs.GuardianSetStruct, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        verifyVM(vm: Structs.VMStruct, overrides?: CallOverrides): Promise<PopulatedTransaction>;
    };
}
