var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { CHAIN_ID_TO_NAME, isChain, CONTRACTS, CHAINS, tryNativeToHexString, ethers_contracts, getSignedVAAWithRetry, parseVaa, } from "../../";
import { BigNumber } from "ethers";
import { getWormholeRelayer, getWormholeRelayerAddress } from "../consts";
import { NodeHttpTransport } from "@improbable-eng/grpc-web-node-http-transport";
import { RelayerPayloadId, parseWormholeRelayerSend, RefundStatus, parseEVMExecutionInfoV1 } from "../structs";
import { getDefaultProvider, printChain, getWormholeRelayerLog, parseWormholeLog, getBlockRange, getWormholeRelayerInfoBySourceSequence, vaaKeyToVaaKeyStruct, getDeliveryProvider } from "./helpers";
export function printWormholeRelayerInfo(info) {
    console.log(stringifyWormholeRelayerInfo(info));
}
export function stringifyWormholeRelayerInfo(info) {
    let stringifiedInfo = "";
    if (info.type == RelayerPayloadId.Delivery && info.deliveryInstruction.targetAddress.toString("hex") !== "0000000000000000000000000000000000000000000000000000000000000000") {
        stringifiedInfo += `Found delivery request in transaction ${info.sourceTransactionHash} on ${info.sourceChain}\n`;
        const numMsgs = info.deliveryInstruction.vaaKeys.length;
        const payload = info.deliveryInstruction.payload.toString("hex");
        if (payload.length > 0) {
            stringifiedInfo += `\nPayload to be relayed (as hex string): 0x${payload}`;
        }
        if (numMsgs > 0) {
            stringifiedInfo += `\nThe following ${numMsgs} wormhole messages (VAAs) were ${payload.length > 0 ? 'also ' : ''}requested to be relayed:\n`;
            stringifiedInfo += info.deliveryInstruction.vaaKeys.map((msgInfo, i) => {
                var _a;
                let result = "";
                result += `(VAA ${i}): `;
                result += `Message from ${msgInfo.chainId ? printChain(msgInfo.chainId) : ""}, with emitter address ${(_a = msgInfo.emitterAddress) === null || _a === void 0 ? void 0 : _a.toString("hex")} and sequence number ${msgInfo.sequence}`;
                return result;
            }).join(",\n");
        }
        if (payload.length == 0 && numMsgs == 0) {
            stringifiedInfo += `\nAn empty payload was requested to be sent`;
        }
        const instruction = info.deliveryInstruction;
        const targetChainName = CHAIN_ID_TO_NAME[instruction.targetChainId];
        stringifiedInfo += `${numMsgs == 0 ? (payload.length == 0 ? '' : '\n\nPayload was requested to be relayed') : '\n\nThese were requested to be sent'} to 0x${instruction.targetAddress.toString("hex")} on ${printChain(instruction.targetChainId)}\n`;
        const totalReceiverValue = (instruction.requestedReceiverValue.add(instruction.extraReceiverValue));
        stringifiedInfo += totalReceiverValue.gt(0)
            ? `Amount to pass into target address: ${totalReceiverValue} wei of ${targetChainName} currency ${instruction.extraReceiverValue.gt(0) ? `${instruction.requestedReceiverValue} requested, ${instruction.extraReceiverValue} additionally paid for` : ""}\n`
            : ``;
        const [executionInfo,] = parseEVMExecutionInfoV1(instruction.encodedExecutionInfo, 0);
        stringifiedInfo += `Gas limit: ${executionInfo.gasLimit} ${targetChainName} gas\n\n`;
        stringifiedInfo += `Refund rate: ${executionInfo.targetChainRefundPerGasUnused} of ${targetChainName} wei per unit of gas unused\n\n`;
        stringifiedInfo += info.targetChainStatus.events
            .map((e, i) => `Delivery attempt ${i + 1}: ${e.transactionHash
            ? ` ${targetChainName} transaction hash: ${e.transactionHash}`
            : ""}\nStatus: ${e.status}\n${e.revertString ? `Failure reason: ${e.gasUsed.eq(executionInfo.gasLimit) ? "Gas limit hit" : e.revertString}\n` : ""}Gas used: ${e.gasUsed.toString()}\nTransaction fee used: ${executionInfo.targetChainRefundPerGasUnused.mul(e.gasUsed).toString()} wei of ${targetChainName} currency\n}`)
            .join("\n");
    }
    else if (info.type == RelayerPayloadId.Delivery && info.deliveryInstruction.targetAddress.toString("hex") === "0000000000000000000000000000000000000000000000000000000000000000") {
        stringifiedInfo += `Found delivery request in transaction ${info.sourceTransactionHash} on ${info.sourceChain}\n`;
        const instruction = info.deliveryInstruction;
        const targetChainName = CHAIN_ID_TO_NAME[instruction.targetChainId];
        stringifiedInfo += `\nA refund of ${instruction.extraReceiverValue} ${targetChainName} wei was requested to be sent to ${targetChainName}, address 0x${info.deliveryInstruction.refundAddress.toString("hex")}`;
        stringifiedInfo += info.targetChainStatus.events
            .map((e, i) => `Delivery attempt ${i + 1}: ${e.transactionHash
            ? ` ${targetChainName} transaction hash: ${e.transactionHash}`
            : ""}\nStatus: ${e.refundStatus == RefundStatus.RefundSent ? "Refund Successful" : "Refund Failed"}`)
            .join("\n");
    }
    return stringifiedInfo;
}
export function sendToEvm(signer, sourceChain, targetChain, targetAddress, payload, gasLimit, overrides, sendOptionalParams) {
    return __awaiter(this, void 0, void 0, function* () {
        const sourceChainId = CHAINS[sourceChain];
        const targetChainId = CHAINS[targetChain];
        const environment = (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.environment) || "MAINNET";
        const wormholeRelayerAddress = getWormholeRelayerAddress(sourceChain, environment);
        const sourceWormholeRelayer = ethers_contracts.IWormholeRelayer__factory.connect(wormholeRelayerAddress, signer);
        const refundLocationExists = (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.refundChainId) !== undefined &&
            (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.refundAddress) !== undefined;
        const defaultDeliveryProviderAddress = yield sourceWormholeRelayer.getDefaultDeliveryProvider();
        // Using the most general 'send' function in IWormholeRelayer
        // Inputs:
        // targetChainId, targetAddress, refundChainId, refundAddress, maxTransactionFee, receiverValue, payload, vaaKeys, 
        // consistencyLevel, deliveryProviderAddress, relayParameters 
        const [deliveryPrice,] = yield sourceWormholeRelayer["quoteEVMDeliveryPrice(uint16,uint256,uint256,address)"](targetChainId, (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.receiverValue) || 0, gasLimit, (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.deliveryProviderAddress) || defaultDeliveryProviderAddress);
        const value = yield ((overrides === null || overrides === void 0 ? void 0 : overrides.value) || 0);
        const totalPrice = deliveryPrice.add((sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.paymentForExtraReceiverValue) || 0);
        if (!totalPrice.eq(value)) {
            throw new Error(`Expected a payment of ${totalPrice.toString()} wei; received ${value.toString()} wei`);
        }
        const tx = sourceWormholeRelayer.sendToEvm(targetChainId, // targetChainId
        targetAddress, // targetAddress
        payload, (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.receiverValue) || 0, // receiverValue 
        (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.paymentForExtraReceiverValue) || 0, // payment for extra receiverValue 
        gasLimit, (refundLocationExists && (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.refundChainId)) || sourceChainId, // refundChainId
        refundLocationExists &&
            (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.refundAddress) &&
            (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.refundAddress) ||
            signer.getAddress(), // refundAddress
        (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.deliveryProviderAddress) || defaultDeliveryProviderAddress, // deliveryProviderAddress
        (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.additionalVaas)
            ? sendOptionalParams.additionalVaas.map((additionalVaa) => ({
                chainId: additionalVaa.chainId || sourceChainId,
                emitterAddress: Buffer.from(tryNativeToHexString(additionalVaa.emitterAddress, "ethereum"), "hex"),
                sequence: BigNumber.from(additionalVaa.sequenceNumber || 0)
            }))
            : [], // vaaKeys
        (sendOptionalParams === null || sendOptionalParams === void 0 ? void 0 : sendOptionalParams.consistencyLevel) || 15, // consistencyLevel
        overrides);
        return tx;
    });
}
export function getPriceAndRefundInfo(sourceChain, targetChain, gasAmount, optionalParams) {
    return __awaiter(this, void 0, void 0, function* () {
        const environment = (optionalParams === null || optionalParams === void 0 ? void 0 : optionalParams.environment) || "MAINNET";
        const sourceChainProvider = (optionalParams === null || optionalParams === void 0 ? void 0 : optionalParams.sourceChainProvider) ||
            getDefaultProvider(environment, sourceChain);
        if (!sourceChainProvider)
            throw Error("No default RPC for this chain; pass in your own provider (as sourceChainProvider)");
        const wormholeRelayerAddress = getWormholeRelayerAddress(sourceChain, environment);
        const sourceWormholeRelayer = ethers_contracts.IWormholeRelayer__factory.connect(wormholeRelayerAddress, sourceChainProvider);
        const deliveryProviderAddress = (optionalParams === null || optionalParams === void 0 ? void 0 : optionalParams.deliveryProviderAddress) ||
            (yield sourceWormholeRelayer.getDefaultDeliveryProvider());
        const targetChainId = CHAINS[targetChain];
        const priceAndRefundInfo = (yield sourceWormholeRelayer["quoteEVMDeliveryPrice(uint16,uint256,uint256,address)"](targetChainId, (optionalParams === null || optionalParams === void 0 ? void 0 : optionalParams.receiverValue) || 0, gasAmount, deliveryProviderAddress));
        return priceAndRefundInfo;
    });
}
export function getPrice(sourceChain, targetChain, gasAmount, optionalParams) {
    return __awaiter(this, void 0, void 0, function* () {
        const priceAndRefundInfo = yield getPriceAndRefundInfo(sourceChain, targetChain, gasAmount, optionalParams);
        return priceAndRefundInfo[0];
    });
}
export function getWormholeRelayerInfo(sourceChain, sourceTransaction, infoRequest) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const environment = (infoRequest === null || infoRequest === void 0 ? void 0 : infoRequest.environment) || "MAINNET";
        const sourceChainProvider = (infoRequest === null || infoRequest === void 0 ? void 0 : infoRequest.sourceChainProvider) ||
            getDefaultProvider(environment, sourceChain);
        if (!sourceChainProvider)
            throw Error("No default RPC for this chain; pass in your own provider (as sourceChainProvider)");
        const receipt = yield sourceChainProvider.getTransactionReceipt(sourceTransaction);
        if (!receipt)
            throw Error("Transaction has not been mined");
        const bridgeAddress = CONTRACTS[environment][sourceChain].core;
        const wormholeRelayerAddress = ((_a = infoRequest === null || infoRequest === void 0 ? void 0 : infoRequest.wormholeRelayerAddresses) === null || _a === void 0 ? void 0 : _a.get(sourceChain)) || getWormholeRelayerAddress(sourceChain, environment);
        if (!bridgeAddress || !wormholeRelayerAddress) {
            throw Error(`Invalid chain ID or network: Chain ${sourceChain}, ${environment}`);
        }
        const deliveryLog = getWormholeRelayerLog(receipt, bridgeAddress, tryNativeToHexString(wormholeRelayerAddress, "ethereum"), (infoRequest === null || infoRequest === void 0 ? void 0 : infoRequest.wormholeRelayerWhMessageIndex)
            ? infoRequest.wormholeRelayerWhMessageIndex
            : 0);
        const { type, parsed } = parseWormholeLog(deliveryLog.log);
        const instruction = parsed;
        const targetChainId = instruction.targetChainId;
        if (!isChain(targetChainId))
            throw Error(`Invalid Chain: ${targetChainId}`);
        const targetChain = CHAIN_ID_TO_NAME[targetChainId];
        const targetChainProvider = ((_b = infoRequest === null || infoRequest === void 0 ? void 0 : infoRequest.targetChainProviders) === null || _b === void 0 ? void 0 : _b.get(targetChain)) ||
            getDefaultProvider(environment, targetChain);
        if (!targetChainProvider) {
            throw Error("No default RPC for this chain; pass in your own provider (as targetChainProvider)");
        }
        const [blockStartNumber, blockEndNumber] = ((_c = infoRequest === null || infoRequest === void 0 ? void 0 : infoRequest.targetChainBlockRanges) === null || _c === void 0 ? void 0 : _c.get(targetChain)) ||
            getBlockRange(targetChainProvider);
        const targetChainStatus = yield getWormholeRelayerInfoBySourceSequence(environment, targetChain, targetChainProvider, sourceChain, BigNumber.from(deliveryLog.sequence), blockStartNumber, blockEndNumber, ((_d = infoRequest === null || infoRequest === void 0 ? void 0 : infoRequest.wormholeRelayerAddresses) === null || _d === void 0 ? void 0 : _d.get(targetChain)) || getWormholeRelayerAddress(targetChain, environment));
        return {
            type: RelayerPayloadId.Delivery,
            sourceChain: sourceChain,
            sourceTransactionHash: sourceTransaction,
            sourceDeliverySequenceNumber: BigNumber.from(deliveryLog.sequence).toNumber(),
            deliveryInstruction: instruction,
            targetChainStatus,
        };
    });
}
export function resendRaw(signer, sourceChain, targetChain, environment, vaaKey, newGasLimit, newReceiverValue, deliveryProviderAddress, overrides) {
    return __awaiter(this, void 0, void 0, function* () {
        const provider = signer.provider;
        if (!provider)
            throw Error("No provider on signer");
        const wormholeRelayer = getWormholeRelayer(sourceChain, environment, signer);
        return wormholeRelayer.resendToEvm(vaaKeyToVaaKeyStruct(vaaKey), CHAINS[targetChain], newReceiverValue, newGasLimit, deliveryProviderAddress, overrides);
    });
}
export function resend(signer, sourceChain, targetChain, environment, vaaKey, newGasLimit, newReceiverValue, deliveryProviderAddress, wormholeRPCs, overrides, isNode) {
    return __awaiter(this, void 0, void 0, function* () {
        const sourceChainId = CHAINS[sourceChain];
        const targetChainId = CHAINS[targetChain];
        const originalVAA = yield getVAA(wormholeRPCs, vaaKey, isNode);
        if (!originalVAA)
            throw Error("orignal VAA not found");
        const originalVAAparsed = parseWormholeRelayerSend(parseVaa(Buffer.from(originalVAA)).payload);
        if (!originalVAAparsed)
            throw Error("orignal VAA not a valid delivery VAA.");
        const [originalExecutionInfo,] = parseEVMExecutionInfoV1(originalVAAparsed.encodedExecutionInfo, 0);
        const originalGasLimit = originalExecutionInfo.gasLimit;
        const originalRefund = originalExecutionInfo.targetChainRefundPerGasUnused;
        const originalReceiverValue = originalVAAparsed.requestedReceiverValue;
        const originalTargetChain = originalVAAparsed.targetChainId;
        if (originalTargetChain != targetChainId) {
            throw Error(`Target chain of original VAA (${originalTargetChain}) does not match target chain of resend (${targetChainId})`);
        }
        if (newReceiverValue < originalReceiverValue) {
            throw Error(`New receiver value too low. Minimum is ${originalReceiverValue.toString()}`);
        }
        if (newGasLimit < originalGasLimit) {
            throw Error(`New gas limit too low. Minimum is ${originalReceiverValue.toString()}`);
        }
        const wormholeRelayer = getWormholeRelayer(sourceChain, environment, signer);
        const deliveryProvider = getDeliveryProvider(deliveryProviderAddress, signer.provider);
        const [deliveryPrice, refundPerUnitGas] = yield wormholeRelayer["quoteEVMDeliveryPrice(uint16,uint256,uint256,address)"](targetChainId, newReceiverValue || 0, newGasLimit, deliveryProviderAddress);
        const value = yield ((overrides === null || overrides === void 0 ? void 0 : overrides.value) || 0);
        if (!deliveryPrice.eq(value)) {
            throw new Error(`Expected a payment of ${deliveryPrice.toString()} wei; received ${value.toString()} wei`);
        }
        if (refundPerUnitGas < originalRefund) {
            throw Error(`New refund per unit gas too low. Minimum is ${originalRefund.toString()}.`);
        }
        return resendRaw(signer, sourceChain, targetChain, environment, vaaKey, newGasLimit, newReceiverValue, deliveryProviderAddress, overrides);
    });
}
export function getVAA(wormholeRPCs, vaaKey, isNode) {
    return __awaiter(this, void 0, void 0, function* () {
        const vaa = yield getSignedVAAWithRetry(wormholeRPCs, vaaKey.chainId, vaaKey.emitterAddress.toString("hex"), vaaKey.sequence.toBigInt().toString(), isNode
            ? {
                transport: NodeHttpTransport(),
            }
            : {}, 2000, 4);
        return vaa.vaaBytes;
    });
}
